@@ -15,7 +15,7 @@
 #include "proctable.h"
 #include "util.h"
 #include "settings-keys.h"
-#include "treeview.h"
+#include "legacy/treeview.h"
 
 #ifndef NI_IDN
 #define NI_IDN 0
@@ -177,10 +177,9 @@ update_openfiles_dialog (GsmTreeView *tree)
     guint i;
 
     pid_t pid = GPOINTER_TO_UINT(static_cast<pid_t*>(g_object_get_data (G_OBJECT (tree), "selected_info")));
-    info = ProcInfo::find(pid);
-
-    if (!info)
-        return;
+    try {
+        info = &GsmApplication::get()->processes.find(pid);
+    } catch (const std::out_of_range&) { return; }
 
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree));
 
@@ -334,7 +333,7 @@ create_single_openfiles_dialog (GtkTreeModel *model, GtkTreePath *path,
 
     label = procman_make_label_for_mmaps_or_ofiles (
         _("_Files opened by process “%s” (PID %u):"),
-        info->name,
+        info->name.c_str(),
         info->pid);
 
     gtk_container_add (GTK_CONTAINER (cmd_grid), GTK_WIDGET (label));
