@@ -63,16 +63,6 @@
 #include <gdk/gdkx.h>
 #endif
 
-ProcInfo::List ProcInfo::all;
-std::map<pid_t, guint64> ProcInfo::cpu_times;
-
-
-ProcInfo* ProcInfo::find(pid_t pid)
-{
-    Iterator it(ProcInfo::all.find(pid));
-    return (it == ProcInfo::all.end() ? NULL : it->second);
-}
-
 static void
 cb_save_tree_state(gpointer, gpointer data)
 {
@@ -199,8 +189,8 @@ cb_refresh_icons (GtkIconTheme *theme, gpointer data)
         g_source_remove (app->timeout);
     }
 
-    for (ProcInfo::Iterator it(ProcInfo::begin()); it != ProcInfo::end(); ++it) {
-        app->pretty_table->set_icon(*(it->second));
+    for (auto& v : app->processes) {
+        app->pretty_table->set_icon(v.second);
     }
 
     cb_timeout(app);
@@ -606,21 +596,6 @@ proctable_new (GsmApplication * const app)
     return proctree;
 }
 
-ProcInfo::~ProcInfo()
-{
-    g_free(this->name);
-    g_free(this->tooltip);
-    g_free(this->arguments);
-    g_free(this->security_context);
-    g_free(this->cgroup_name);
-    // The following are allocated inside of the sd_pid_get_*
-    // functions using malloc(). Free with free() instead of g_free()
-    // to insure proper clean up.
-    free(this->unit);
-    free(this->session);
-    free(this->seat);
-}
-
 static void
 get_process_name (ProcInfo *info,
                   const gchar *cmd, const GStrv args)
@@ -641,8 +616,43 @@ get_process_name (ProcInfo *info,
             g_free(basename);
         }
     }
+    info->name = cmd;
+}
 
-    info->name = g_strdup (cmd);
+std::pair<ProcList::Iterator, bool> ProcList::emplace(pid_t pid) {
+    std::lock_guard<std::mutex> lg(data_lock);
+    auto res = data.emplace(pid, pid);
+    if (res.second) {
+        auto& entry = res.first->second;
+        glibtop_proc_state procstate;
+        glibtop_proc_args procargs;
+        gchar** arguments;
+        glibtop_get_proc_state (&procstate, pid);
+        arguments = glibtop_get_proc_argv (&procargs, pid, 0);
+        /* FIXME : wrong. name and arguments may change with exec* */
+        get_process_name (&entry, procstate.cmd, static_cast<const GStrv>(arguments));
+        std::string tooltip = make_string(g_strjoinv(" ", arguments));
+        if (tooltip.empty())
+            tooltip = procstate.cmd;
+        entry.tooltip = g_markup_escape_text(tooltip.c_str(), -1);
+        entry.arguments = g_strescape(tooltip.c_str(), "\\\"");
+        g_strfreev(arguments);
+        get_process_selinux_context (&entry);
+        get_process_cgroup_info(entry);
+        get_process_systemd_info(&entry);
+        glibtop_proc_time proctime;
+        glibtop_get_proc_time (&proctime, pid);
+        guint64 cpu_time = proctime.rtime;
+        auto it = cpu_times.find(pid);
+        if (it != std::end(cpu_times))
+        {
+            if (proctime.rtime >= it->second)
+                cpu_time = it->second;
+        }
+        entry.cpu_time = cpu_time;
+        entry.start_time = proctime.start_time;
+    }
+    return res;
 }
 
 std::string
@@ -763,11 +773,11 @@ update_info_mutable_cols(ProcInfo *info)
     tree_store_update(model, &info->node, COL_START_TIME, info->start_time);
     tree_store_update(model, &info->node, COL_NICE, info->nice);
     tree_store_update(model, &info->node, COL_MEM, info->mem);
-    tree_store_update(model, &info->node, COL_WCHAN, info->wchan);
-    tree_store_update(model, &info->node, COL_CGROUP, info->cgroup_name);
-    tree_store_update(model, &info->node, COL_UNIT, info->unit);
-    tree_store_update(model, &info->node, COL_SESSION, info->session);
-    tree_store_update(model, &info->node, COL_SEAT, info->seat);
+    tree_store_update(model, &info->node, COL_WCHAN, info->wchan.c_str());
+    tree_store_update(model, &info->node, COL_CGROUP, info->cgroup_name.c_str());
+    tree_store_update(model, &info->node, COL_UNIT, info->unit.c_str());
+    tree_store_update(model, &info->node, COL_SESSION, info->session.c_str());
+    tree_store_update(model, &info->node, COL_SEAT, info->seat.c_str());
     tree_store_update(model, &info->node, COL_OWNER, info->owner.c_str());
 }
 
@@ -786,7 +796,7 @@ insert_info_to_tree (ProcInfo *info, GsmApplication *app, bool forced = false)
         ProcInfo *parent = 0;
 
         if (not forced)
-            parent = ProcInfo::find(info->ppid);
+            try { parent = &app->processes.find(info->ppid); } catch (const std::out_of_range&) { parent = nullptr; }
 
         if (parent) {
             GtkTreePath *parent_node = gtk_tree_model_get_path(model, &parent->node);
@@ -817,11 +827,11 @@ insert_info_to_tree (ProcInfo *info, GsmApplication *app, bool forced = false)
 
     gtk_tree_store_set (GTK_TREE_STORE (model), &info->node,
                         COL_POINTER, info,
-                        COL_NAME, info->name,
-                        COL_ARGS, info->arguments,
-                        COL_TOOLTIP, info->tooltip,
+                        COL_NAME, info->name.c_str(),
+                        COL_ARGS, info->arguments.c_str(),
+                        COL_TOOLTIP, info->tooltip.c_str(),
                         COL_PID, info->pid,
-                        COL_SECURITYCONTEXT, info->security_context,
+                        COL_SECURITYCONTEXT, info->security_context.c_str(),
                         -1);
 
     app->pretty_table->set_icon(*info);
@@ -835,36 +845,44 @@ insert_info_to_tree (ProcInfo *info, GsmApplication *app, bool forced = false)
 template<typename List>
 static void
 remove_info_from_tree (GsmApplication *app, GtkTreeModel *model,
-                       ProcInfo *current, List &orphans, unsigned lvl = 0)
+                       ProcInfo& current, List &orphans, unsigned lvl = 0)
 {
     GtkTreeIter child_node;
 
-    if (std::find(orphans.begin(), orphans.end(), current) != orphans.end()) {
-        procman_debug("[%u] %d already removed from tree", lvl, int(current->pid));
+    if (std::find(orphans.begin(), orphans.end(), &current) != orphans.end()) {
+        procman_debug("[%u] %d already removed from tree", lvl, int(current.pid));
         return;
     }
 
-    procman_debug("[%u] pid %d, %d children", lvl, int(current->pid),
-                  gtk_tree_model_iter_n_children(model, &current->node));
+    procman_debug("[%u] pid %d, %d children", lvl, int(current.pid),
+                  gtk_tree_model_iter_n_children(model, &current.node));
 
     // it is not possible to iterate&erase over a treeview so instead we
     // just pop one child after another and recursively remove it and
     // its children
 
-    while (gtk_tree_model_iter_children(model, &child_node, &current->node)) {
+    while (gtk_tree_model_iter_children(model, &child_node, &current.node)) {
         ProcInfo *child = 0;
         gtk_tree_model_get(model, &child_node, COL_POINTER, &child, -1);
-        remove_info_from_tree(app, model, child, orphans, lvl + 1);
+        remove_info_from_tree(app, model, *child, orphans, lvl + 1);
     }
 
-    g_assert(not gtk_tree_model_iter_has_child(model, &current->node));
+    g_assert(not gtk_tree_model_iter_has_child(model, &current.node));
 
-    orphans.push_back(current);
-    gtk_tree_store_remove(GTK_TREE_STORE(model), &current->node);
-    procman::poison(current->node, 0x69);
+    orphans.push_back(&current);
+    gtk_tree_store_remove(GTK_TREE_STORE(model), &current.node);
+    procman::poison(current.node, 0x69);
 }
 
 
+static std::string
+get_proc_kernel_wchan(glibtop_proc_kernel& obj) {
+    char buf[40] = {0};
+    g_strlcpy(buf, obj.wchan, sizeof(buf));
+    buf[sizeof(buf)-1] = '\0';
+    return buf;
+}
+
 static void
 update_info (GsmApplication *app, ProcInfo *info)
 {
@@ -874,7 +892,7 @@ update_info (GsmApplication *app, ProcInfo *info)
     glibtop_proc_kernel prockernel;
 
     glibtop_get_proc_kernel(&prockernel, info->pid);
-    g_strlcpy(info->wchan, prockernel.wchan, sizeof info->wchan);
+    info->wchan = get_proc_kernel_wchan(prockernel);
 
     glibtop_get_proc_state (&procstate, info->pid);
     info->status = procstate.state;
@@ -897,7 +915,7 @@ update_info (GsmApplication *app, ProcInfo *info)
     if (not app->config.solaris_mode)
         info->pcpu *= app->config.num_cpus;
 
-    ProcInfo::cpu_times[info->pid] = info->cpu_time = proctime.rtime;
+    app->processes.cpu_times[info->pid] = info->cpu_time = proctime.rtime;
     info->nice = procuid.nice;
 
     // set the ppid only if one can exist
@@ -918,52 +936,10 @@ update_info (GsmApplication *app, ProcInfo *info)
 ProcInfo::ProcInfo(pid_t pid)
     : node(),
       pixbuf(),
-      tooltip(NULL),
-      name(NULL),
-      arguments(NULL),
-      security_context(NULL),
       pid(pid),
       ppid(-1),
       uid(-1)
 {
-    ProcInfo * const info = this;
-    glibtop_proc_state procstate;
-    glibtop_proc_time proctime;
-    glibtop_proc_args procargs;
-    gchar** arguments;
-
-    glibtop_get_proc_state (&procstate, pid);
-    glibtop_get_proc_time (&proctime, pid);
-    arguments = glibtop_get_proc_argv (&procargs, pid, 0);
-
-    /* FIXME : wrong. name and arguments may change with exec* */
-    get_process_name (info, procstate.cmd, static_cast<const GStrv>(arguments));
-
-    std::string tooltip = make_string(g_strjoinv(" ", arguments));
-    if (tooltip.empty())
-        tooltip = procstate.cmd;
-
-    info->tooltip = g_markup_escape_text(tooltip.c_str(), -1);
-
-    info->arguments = g_strescape(tooltip.c_str(), "\\\"");
-    g_strfreev(arguments);
-
-    guint64 cpu_time = proctime.rtime;
-    std::map<pid_t, guint64>::iterator it(ProcInfo::cpu_times.find(pid));
-    if (it != ProcInfo::cpu_times.end())
-    {
-        if (proctime.rtime >= it->second)
-            cpu_time = it->second;
-    }
-    info->cpu_time = cpu_time;
-    info->start_time = proctime.start_time;
-
-    get_process_selinux_context (info);
-    info->cgroup_name = NULL;
-    get_process_cgroup_info(*info);
-
-    info->unit = info->session = info->seat = NULL;
-    get_process_systemd_info(info);
 }
 
 static void
@@ -978,12 +954,12 @@ refresh_list (GsmApplication *app, const pid_t* pid_list, const guint n)
     guint i;
 
     // Add or update processes in the process list
-    for(i = 0; i < n; ++i) {
-        ProcInfo *info = ProcInfo::find(pid_list[i]);
-
-        if (!info) {
-            info = new ProcInfo(pid_list[i]);
-            ProcInfo::all[info->pid] = info;
+    for (i = 0; i < n; ++i) {
+        ProcInfo* info;
+        try {
+            info = &app->processes.find(pid_list[i]);
+        } catch (const std::out_of_range&) {
+            info = &app->processes.emplace(pid_list[i]).first->second;
             addition.push_back(info);
         }
 
@@ -997,22 +973,17 @@ refresh_list (GsmApplication *app, const pid_t* pid_list, const guint n)
 
     const std::set<pid_t> pids(pid_list, pid_list + n);
 
-    ProcInfo::Iterator it(ProcInfo::begin());
-
-    while (it != ProcInfo::end()) {
-        ProcInfo * const info = it->second;
-        ProcInfo::Iterator next(it);
-        ++next;
-
-        if (pids.find(info->pid) == pids.end()) {
-            procman_debug("ripping %d", info->pid);
+    auto it = std::begin(app->processes);
+    while (it != std::end(app->processes)) {
+        auto& info = it->second;
+        if (pids.find(info.pid) == pids.end()) {
+            procman_debug("ripping %d", info.pid);
             remove_info_from_tree(app, model, info, addition);
-            addition.remove(info);
-            ProcInfo::all.erase(it);
-            delete info;
+            addition.remove(&info);
+            it = app->processes.erase(it);
+        } else {
+            ++it;
         }
-
-        it = next;
     }
 
     // INVARIANT
@@ -1064,7 +1035,8 @@ refresh_list (GsmApplication *app, const pid_t* pid_list, const guint n)
                     continue;
                 }
 
-                ProcInfo *parent = ProcInfo::find((*it)->ppid);
+                ProcInfo* parent;
+                try { parent = &app->processes.find((*it)->ppid); } catch (const std::out_of_range& e) { parent = nullptr; }
                 // if the parent is unreachable
                 if (not parent) {
                     // or std::find(addition.begin(), addition.end(), parent) == addition.end()) {
@@ -1080,13 +1052,11 @@ refresh_list (GsmApplication *app, const pid_t* pid_list, const guint n)
     }
     else {
         // don't care of the tree
-        for (ProcList::iterator it(addition.begin()); it != addition.end(); ++it)
-            insert_info_to_tree(*it, app);
+        for (auto& v : addition) insert_info_to_tree(v, app);
     }
 
 
-    for (ProcInfo::Iterator it(ProcInfo::begin()); it != ProcInfo::end(); ++it)
-        update_info_mutable_cols(it->second);
+    for (auto& v : app->processes) update_info_mutable_cols(&v.second);
 }
 
 void
@@ -1158,10 +1128,7 @@ proctable_update (GsmApplication *app)
 void
 proctable_free_table (GsmApplication * const app)
 {
-    for (ProcInfo::Iterator it(ProcInfo::begin()); it != ProcInfo::end(); ++it)
-        delete it->second;
-
-    ProcInfo::all.clear();
+    app->processes.clear();
 }
 
 void
